package com.qr.flutter_toolkit.camera;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.hardware.Camera;
import android.os.Build;

import androidx.annotation.NonNull;

import com.qr.flutter_toolkit.utils.BitmapUtils;
import com.qr.flutter_toolkit.utils.CameraUtils;
import com.qr.flutter_toolkit.utils.PermissionUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.Log;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.view.TextureRegistry;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;

public class CameraHandler implements MethodChannel.MethodCallHandler {
    private final static String TAG = CameraHandler.class.getSimpleName();
    private final PluginRegistry.Registrar registrar;
    private TextureRegistry.SurfaceTextureEntry surfaceTextureEntry;

    private List<String> manifestPermissions = null;

    private Camera camera = null;
    private int cameraId = Camera.CameraInfo.CAMERA_FACING_BACK;

    // 保留最后的配置
    private int previewWidth;
    private int previewHeight;
    private int pictureWidth;
    private int pictureHeight;

    public CameraHandler(PluginRegistry.Registrar registrar) {
        this.registrar = registrar;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
        Log.d(TAG, "Method: " + methodCall.method);

        if (!hasPermissionInManifest(Manifest.permission.CAMERA)) {
            result.error("Camera Failure", "Camera Permission Not In Manifest", null);
            return;
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (registrar.context().checkSelfPermission(Manifest.permission.CAMERA) == PackageManager.PERMISSION_DENIED) {
                result.error("Camera Failure", "No Camera Permission", null);
                return;
            }
        }

        switch (methodCall.method) {
            case "getAvailableCameras":
                result.success(CameraUtils.getAvailableCameras());
                break;
            case "startPreview":
                getPreviewCameraConfig(methodCall);
                try {
                    result.success(startPreview());
                } catch (Exception e) {
                    stopPreview();
                    result.error("Camera Fail", "Start Preview Fail! Reason: " + e.getMessage(), null);
                }
                break;
            case "stopPreview":
                // 释放掉该SurfaceTextureEntity
                if (surfaceTextureEntry != null) {
                    surfaceTextureEntry.release();
                    surfaceTextureEntry = null;
                }
                stopPreview();
                result.success(null);
                break;
            case "switchCamera":
                if (cameraId == Camera.CameraInfo.CAMERA_FACING_BACK) {
                    cameraId = Camera.CameraInfo.CAMERA_FACING_FRONT;
                } else {
                    cameraId = Camera.CameraInfo.CAMERA_FACING_BACK;
                }

                getPreviewCameraConfig(methodCall);

                try {
                    result.success(startPreview());
                } catch (Exception e) {
                    stopPreview();
                    result.error("Camera Fail", "Start Preview Fail! Reason: " + e.getMessage(), null);
                }
                break;
            case "takePicture":
                takePicture(methodCall, result);
                break;
            case "startRecording":
                if (camera == null) {
                    result.error("Camera Failure", "Start Recording Failure! Camera Not Open", null);
                    return;
                }
                try {
                    Camera.Size previewSize = camera.getParameters().getPreviewSize();
                    camera.addCallbackBuffer(new byte[previewSize.width * previewSize.height * 3 / 2]);
                    camera.setPreviewCallbackWithBuffer(new Camera.PreviewCallback() {
                        @Override
                        public void onPreviewFrame(byte[] data, Camera camera) {
                            // TODO: handle 帧数据
                            camera.addCallbackBuffer(data);
                        }
                    });
                    result.success(null);
                } catch (Exception e) {
                    result.error("Camera Failure", "Start Recording Failure! Reason: " + e.getMessage(), null);
                }
                break;
            case "stopRecording":
                if (camera == null) {
                    result.error("Camera Failure", "Stop Recording Failure! Camera Not Open", null);
                    return;
                }
                try {
                    camera.setPreviewCallbackWithBuffer(null);
                    result.success(null);
                } catch (Exception e) {
                    result.error("Camera Failure", "Stop Recording Failure! Reason: " + e.getMessage(), null);
                }
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void getPreviewCameraConfig(MethodCall methodCall) {
        Integer previewWidth = methodCall.argument("previewWidth");
        Integer previewHeight = methodCall.argument("previewHeight");
        Integer pictureWidth = methodCall.argument("pictureWidth");
        Integer pictureHeight = methodCall.argument("pictureHeight");

        if (previewWidth != null && previewHeight != null) {
            this.previewWidth = previewWidth;
            this.previewHeight = previewHeight;
        } else {
            this.previewWidth = 1920;
            this.previewHeight = 1080;
        }

        if (pictureWidth != null && pictureHeight != null) {
            this.pictureWidth = pictureWidth;
            this.pictureHeight = pictureHeight;
        } else {
            this.pictureWidth = 1920;
            this.pictureHeight = 1080;
        }
    }

    private Map<String, Object> startPreview() throws Exception {
        stopPreview();

        camera = Camera.open(cameraId);
        Camera.Parameters parameters = camera.getParameters();

        // 设置预览尺寸
        List<Camera.Size> supportedPreviewSizes = parameters.getSupportedPreviewSizes();
        Camera.Size bestPreviewSizeValue = CameraUtils.findBestPreviewSizeValue(supportedPreviewSizes,
                previewWidth, previewHeight);
        if (bestPreviewSizeValue == null) {
            bestPreviewSizeValue = supportedPreviewSizes.get(0);
        }
        Log.d(TAG, "bestPreviewSize Width: " + bestPreviewSizeValue.width + " Height: " + bestPreviewSizeValue.height);
        parameters.setPreviewSize(bestPreviewSizeValue.width, bestPreviewSizeValue.height);

        // 设置拍照尺寸 如果没有设置尺寸 则以1920*1080拍照
        Camera.Size bestPictureSizeValue = CameraUtils.findBestPreviewSizeValue(supportedPreviewSizes,
                pictureWidth, pictureHeight);
        if (bestPictureSizeValue == null) {
            bestPictureSizeValue = bestPreviewSizeValue;
        }
        Log.d(TAG, "bestPictureSize Width: " + bestPictureSizeValue.width + " Height: " + bestPictureSizeValue.height);
        parameters.setPictureSize(bestPictureSizeValue.width, bestPictureSizeValue.height);

        // 设置预览FPS 以最好FPS进行预览
        int[] bestFpsRange = new int[2];
        List<int[]> supportedPreviewFpsRange = parameters.getSupportedPreviewFpsRange();
        for (int[] ints : supportedPreviewFpsRange) {
            if (ints[1] > bestFpsRange[1]) {
                // 最大分辨率大于当前分辨率
                bestFpsRange[0] = ints[0];
                bestFpsRange[1] = ints[1];
            } else if (ints[1] == bestFpsRange[1]) {
                // 最大分辨率相等
                if (ints[0] > bestFpsRange[0]) {
                    bestFpsRange[0] = ints[0];
                }
            }
        }

        Log.d(TAG, "bestFpsRange minFps: " + bestFpsRange[0] + " maxFps: " + bestFpsRange[1]);
        parameters.setPreviewFpsRange(bestFpsRange[0], bestFpsRange[1]);

        camera.setParameters(parameters);
        CameraUtils.setCameraDisplayOrientation(camera,
                cameraId,
                registrar.activity().getWindowManager().getDefaultDisplay().getRotation());

        if (surfaceTextureEntry == null) {
            surfaceTextureEntry = registrar.textures().createSurfaceTexture();
        }
        camera.setPreviewTexture(surfaceTextureEntry.surfaceTexture());
        camera.startPreview();

        Map<String, Object> map = new HashMap<>();
        map.put("cameraId", cameraId);
        map.put("textureId", surfaceTextureEntry.id());
        map.put("previewWidth", bestPreviewSizeValue.width);
        map.put("previewHeight", bestPreviewSizeValue.height);
        map.put("pictureWidth", bestPictureSizeValue.width);
        map.put("pictureHeight", bestPictureSizeValue.height);
        map.put("minFps", bestFpsRange[0]);
        map.put("maxFps", bestFpsRange[1]);

        return map;
    }

    private void stopPreview() {
        if (camera != null) {
            try {
                camera.stopPreview();
                camera.release();
            } catch (Exception e) {
                Log.d(TAG, "Stop Camera Fail! Reason: " + e.getMessage());
            } finally {
                camera = null;
            }
        }
    }

    private void takePicture(MethodCall methodCall, MethodChannel.Result result) {
        String dirPath = methodCall.argument("dirPath");
        if (dirPath == null) {
            result.error("Camera Failure", "takePicture Failure! dirPath == null", null);
            return;
        }
        String fileName = methodCall.argument("fileName");
        if (fileName == null) {
            result.error("Camera Failure", "takePicture Failure! fileName == null", null);
            return;
        }

        if (camera == null) {
            result.error("Camera Failure", "takePicture Failure! Camera Not Open", null);
            return;
        }

        camera.takePicture(null, null, new Camera.PictureCallback() {
            @Override
            public void onPictureTaken(byte[] data, Camera camera) {
                Schedulers.newThread().scheduleDirect(new Runnable() {
                    @Override
                    public void run() {
                        if (data == null) {
                            AndroidSchedulers.mainThread().scheduleDirect(new Runnable() {
                                @Override
                                public void run() {
                                    result.error("Camera Failure", "takePicture Failure! data == null", null);
                                }
                            });
                            return;
                        }

                        try {
                            File dir = new File(dirPath);
                            if (!dir.exists()) {
                                dir.mkdirs();
                            }

                            File file = new File(dir, fileName);

                            FileOutputStream fileOutputStream = new FileOutputStream(file);

                            Bitmap bitmap = BitmapFactory.decodeByteArray(data, 0, data.length);
                            Bitmap resultBtm;
                            if (CameraHandler.this.cameraId == Camera.CameraInfo.CAMERA_FACING_FRONT) {
                                resultBtm = BitmapUtils.mirror(BitmapUtils.rotate(bitmap, 270f));
                            } else {
                                resultBtm = BitmapUtils.rotate(bitmap, 90f);
                            }

                            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                            resultBtm.compress(Bitmap.CompressFormat.JPEG, 100, byteArrayOutputStream);
                            byte[] bytes = byteArrayOutputStream.toByteArray();

                            fileOutputStream.write(bytes);
                            AndroidSchedulers.mainThread().scheduleDirect(new Runnable() {
                                @Override
                                public void run() {
                                    result.success(dirPath);
                                }
                            });
                        } catch (Exception e) {
                            e.printStackTrace();
                            AndroidSchedulers.mainThread().scheduleDirect(new Runnable() {
                                @Override
                                public void run() {
                                    result.error("Camera Failure", "TakePicture Failure! reason: " + e.getMessage(), null);
                                }
                            });
                        }
                    }
                });
            }
        });
    }

    private boolean hasPermissionInManifest(String permission) {
        if (manifestPermissions == null) {
            Context context = registrar.activeContext();
            if (context == null) {
                Log.d(TAG, "Unable to detect current Activity or App Context.");
                return false;
            }
            manifestPermissions = PermissionUtils.getManifestPermissions(context);
        }
        return manifestPermissions.contains(permission);
    }

    public void onResumed() {

    }

    public void onPaused() {
        stopPreview();
    }
}
