package com.qr.flutter_toolkit.gps_location;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Build;
import android.os.Bundle;
import android.os.Looper;

import androidx.annotation.NonNull;

import java.util.HashMap;
import java.util.Map;

import io.flutter.Log;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class GPSLocationHandler implements MethodChannel.MethodCallHandler, EventChannel.StreamHandler {
    private final PluginRegistry.Registrar registrar;
    private LocationListener listener = null;
    private EventChannel.EventSink eventSink = null;

    public GPSLocationHandler(PluginRegistry.Registrar registrar) {
        this.registrar = registrar;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (registrar.context().checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_DENIED ||
                    registrar.context().checkSelfPermission(Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_DENIED) {
                result.error("GPSLocation fail", "no location permissions", null);
                return;
            }
        }

        LocationManager locationManager = getLocationManager();
        if (locationManager == null) {
            result.error("GPSLocation fail", "locationManager == null", null);
            return;
        }

        if (!locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            result.error("GPSLocation fail", "GPS is not available", null);
            return;
        }

        switch (methodCall.method) {
            case "getLastKnownGPSLocation":
                getLastKnownGPSLocation(locationManager, result);
                break;
            case "getCurrentGPSLocation":
                getCurrentGPSLocation(locationManager, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onListen(Object o, EventChannel.EventSink eventSink) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (registrar.context().checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_DENIED ||
                    registrar.context().checkSelfPermission(Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_DENIED) {
                eventSink.error("GPSLocation fail", "no location permissions", null);
                eventSink.endOfStream();
                return;
            }
        }

        LocationManager locationManager = getLocationManager();
        if (locationManager == null) {
            eventSink.error("GPSLocation fail", "locationManager == null", null);
            eventSink.endOfStream();
            return;
        }

        if (!locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            eventSink.error("GPSLocation fail", "GPS is not available", null);
            eventSink.endOfStream();
            return;
        }

        if (listener != null) {
            // 应该不会调用该函数 在onCancel函数会比onListen先调用
            locationManager.removeUpdates(listener);
            listener = null;
        }

        if (this.eventSink != null) {
            // 应该不会调用该函数 在onCancel函数会比onListen先调用
            this.eventSink.error("GPSLocation fail", "new eventSink", null);
            this.eventSink.endOfStream();
            this.eventSink = null;
        }
        this.eventSink = eventSink;

        // 开始监听
        listener = new LocationListener() {
            @Override
            public void onLocationChanged(Location location) {
                GPSLocationHandler.this.eventSink.success(locationToMap(location));
            }

            @Override
            public void onStatusChanged(String provider, int status, Bundle extras) {
                if (status == LocationProvider.OUT_OF_SERVICE) {
                    onProviderDisabled(provider);
                }
            }

            @Override
            public void onProviderEnabled(String provider) {
                Log.d(GPSLocationHandler.class.getSimpleName(), provider + ": Enabled");
            }

            @Override
            public void onProviderDisabled(String provider) {
                Log.d(GPSLocationHandler.class.getSimpleName(), provider + ": Disabled");
                GPSLocationHandler.this.eventSink.error("GPSLocation Fail", provider + ": Disabled", null);
            }
        };

        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
                10000,
                0,
                listener,
                Looper.getMainLooper());
    }

    @Override
    public void onCancel(Object o) {
        if (listener != null) {
            getLocationManager().removeUpdates(listener);
            listener = null;
        }

        if (eventSink != null) {
            this.eventSink.error("GPSLocation fail", "onCancel", null);
            eventSink.endOfStream();
            eventSink = null;
        }
    }

    private void getLastKnownGPSLocation(LocationManager locationManager, MethodChannel.Result result) {
        Location lastKnownLocation = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
        result.success(locationToMap(lastKnownLocation));
    }

    private void getCurrentGPSLocation(LocationManager locationManager, final MethodChannel.Result result) {
        locationManager.requestSingleUpdate(LocationManager.GPS_PROVIDER,
                new OnceLocationListener(locationManager, result),
                Looper.getMainLooper());
    }

    private LocationManager getLocationManager() {
        return (LocationManager) registrar.context().getSystemService(Context.LOCATION_SERVICE);
    }

    private static Map<String, Object> locationToMap(Location location) {
        if (location == null) {
            return null;
        }

        Map<String, Object> position = new HashMap<>();
        position.put("latitude", location.getLatitude());
        position.put("longitude", location.getLongitude());
        position.put("timestamp", location.getTime());

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
            position.put("mocked", location.isFromMockProvider());
        }

        if (location.hasAltitude())
            position.put("altitude", location.getAltitude());
        if (location.hasAccuracy())
            position.put("accuracy", (double) location.getAccuracy());
        if (location.hasBearing())
            position.put("bearing", (double) location.getBearing());
        if (location.hasSpeed())
            position.put("speed", (double) location.getSpeed());
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && location.hasSpeedAccuracy())
            position.put("speedAccuracy", (double) location.getSpeedAccuracyMetersPerSecond());

        return position;
    }


    public static class OnceLocationListener implements LocationListener {
        private LocationManager locationManager;
        private MethodChannel.Result result;

        public OnceLocationListener(LocationManager locationManager, MethodChannel.Result result) {
            this.locationManager = locationManager;
            this.result = result;
        }

        @Override
        public void onLocationChanged(Location location) {
            result.success(locationToMap(location));
            locationManager.removeUpdates(this);
            result = null;
            locationManager = null;
        }

        @Override
        public void onStatusChanged(String provider, int status, Bundle extras) {

        }

        @Override
        public void onProviderEnabled(String provider) {

        }

        @Override
        public void onProviderDisabled(String provider) {

        }
    }
}
