package com.qr.flutter_toolkit.permission;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.PowerManager;
import android.provider.Settings;

import androidx.annotation.NonNull;

import com.qr.flutter_toolkit.utils.PermissionUtils;

import java.util.ArrayList;
import java.util.List;

import io.flutter.Log;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class PermissionHandler implements MethodChannel.MethodCallHandler {
    private final String TAG = PermissionHandler.class.getSimpleName();

    public static final int PERMISSION_CODE = 24;
    public static final int PERMISSION_CODE_IGNORE_BATTERY_OPTIMIZATIONS = 5672353;
    public static final int PERMISSION_CODE_SYSTEM_ALERT_WINDOW = 5672354;
    public static final int PERMISSION_CODE_REQUEST_INSTALL_PACKAGES = 5672355;

    // 权限组
    private static final int PERMISSION_GROUP_CALENDAR = 0;
    private static final int PERMISSION_GROUP_CAMERA = 1;
    private static final int PERMISSION_GROUP_CONTACTS = 2;
    private static final int PERMISSION_GROUP_LOCATION = 3;
    private static final int PERMISSION_GROUP_MICROPHONE = 4;
    private static final int PERMISSION_GROUP_PHONE = 5;
    private static final int PERMISSION_GROUP_SENSORS = 6;
    private static final int PERMISSION_GROUP_SMS = 7;
    private static final int PERMISSION_GROUP_STORAGE = 8;
    private static final int PERMISSION_GROUP_IGNORE_BATTERY_OPTIMIZATIONS = 9;
    private static final int PERMISSION_GROUP_SYSTEM_ALERT_WINDOW = 10;
    private static final int PERMISSION_GROUP_REQUEST_INSTALL_PACKAGES = 11;

    // 权限状态
    private static final int PERMISSION_STATUS_DENIED = 0;
    private static final int PERMISSION_STATUS_GRANTED = 1;

    private final PluginRegistry.Registrar registrar;

    private List<String> manifestPermissions = null;
    private MethodChannel.Result result = null;

    public PermissionHandler(PluginRegistry.Registrar registrar) {
        this.registrar = registrar;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
        switch (methodCall.method) {
            case "checkPermissionsStatus":
                checkPermissionsStatus(methodCall, result);
                break;
            case "requestPermission":
                requestPermission(methodCall, result);
                break;
            case "shouldShowRequestPermissionRationale":
                shouldShowRequestPermissionRationale(methodCall, result);
                break;
            case "openAppSetting":
                openAppSetting(result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }


    private void checkPermissionsStatus(MethodCall methodCall, MethodChannel.Result result) {
        Context context = registrar.activeContext();
        if (context == null) {
            Log.d(TAG, "Unable to detect current Activity or App Context.");
            result.error("checkPermission Error", "Unable to detect current Activity or App Context", null);
            return;
        }

        int group = methodCall.arguments();

        List<String> manifestPermissions = getManifestPermissions(group);
        if (manifestPermissions == null) {
            Log.d(TAG, "No android specific permissions needed for: " + group);
            result.success(PERMISSION_STATUS_GRANTED);
            return;
        }

        if (manifestPermissions.isEmpty()) {
            Log.d(TAG, "No permissions found in manifest for: " + group);
            result.success(PERMISSION_STATUS_DENIED);
            return;
        }

        if (group == PERMISSION_GROUP_IGNORE_BATTERY_OPTIMIZATIONS) {
            String packageName = context.getPackageName();
            PowerManager powerManager = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (powerManager.isIgnoringBatteryOptimizations(packageName)) {
                    result.success(PERMISSION_STATUS_GRANTED);
                } else {
                    result.success(PERMISSION_STATUS_DENIED);
                }
            } else {
                result.success(PERMISSION_STATUS_DENIED);
            }
            return;
        }

        if (group == PERMISSION_GROUP_SYSTEM_ALERT_WINDOW) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (Settings.canDrawOverlays(context)) {
                    result.success(PERMISSION_STATUS_GRANTED);
                } else {
                    result.success(PERMISSION_STATUS_DENIED);
                }
            } else {
                result.success(PERMISSION_STATUS_GRANTED);
            }
            return;
        }

        if (group == PERMISSION_GROUP_REQUEST_INSTALL_PACKAGES) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                if (context.getPackageManager().canRequestPackageInstalls()) {
                    result.success(PERMISSION_STATUS_GRANTED);
                } else {
                    result.success(PERMISSION_STATUS_DENIED);
                }
            } else {
                result.success(PERMISSION_STATUS_GRANTED);
            }
            return;
        }


        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            for (String permission : manifestPermissions) {
                if (context.checkSelfPermission(permission) == PackageManager.PERMISSION_DENIED) {
                    result.success(PERMISSION_STATUS_DENIED);
                    return;
                }
            }
        }

        result.success(PERMISSION_STATUS_GRANTED);
    }

    private void requestPermission(MethodCall methodCall, MethodChannel.Result result) {
        if (this.result != null) {
            result.error("requestPermission Error", "Repeat Request", null);
            return;
        }

        this.result = result;

        Activity activity = registrar.activity();
        if (activity == null) {
            Log.d(TAG, "Unable to detect current Activity");
            this.result.error("requestPermission Error", "Unable to detect current Activity!", null);
            this.result = null;
            return;
        }

        int group = methodCall.arguments();

        List<String> manifestPermissions = getManifestPermissions(group);
        if (manifestPermissions == null) {
            Log.d(TAG, "No android specific permissions needed for: " + group);
            this.result.success(PERMISSION_STATUS_GRANTED);
            this.result = null;
            return;
        }

        if (manifestPermissions.isEmpty()) {
            Log.d(TAG, "No permissions found in manifest for: " + group);
            this.result.success(PERMISSION_STATUS_DENIED);
            this.result = null;
            return;
        }

        if (group == PERMISSION_GROUP_IGNORE_BATTERY_OPTIMIZATIONS) {
            String packageName = activity.getPackageName();
            PowerManager powerManager = (PowerManager) activity.getSystemService(Context.POWER_SERVICE);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (powerManager.isIgnoringBatteryOptimizations(packageName)) {
                    this.result.success(PERMISSION_STATUS_GRANTED);
                    this.result = null;
                } else {
                    Intent intent = new Intent();
                    intent.setAction(Settings.ACTION_REQUEST_IGNORE_BATTERY_OPTIMIZATIONS);
                    intent.setData(Uri.parse("package:" + packageName));
                    activity.startActivityForResult(intent, PERMISSION_CODE_IGNORE_BATTERY_OPTIMIZATIONS);
                }
            } else {
                this.result.success(PERMISSION_STATUS_DENIED);
                this.result = null;
            }

            return;
        }

        if (group == PERMISSION_GROUP_SYSTEM_ALERT_WINDOW) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                if (Settings.canDrawOverlays(activity)) {
                    this.result.success(PERMISSION_STATUS_GRANTED);
                    this.result = null;
                } else {
                    Intent intent = new Intent(Settings.ACTION_MANAGE_OVERLAY_PERMISSION);
                    intent.setData(Uri.parse("package:" + activity.getPackageName()));
                    activity.startActivityForResult(intent, PERMISSION_CODE_SYSTEM_ALERT_WINDOW);
                }
            } else {
                this.result.success(PERMISSION_STATUS_GRANTED);
                this.result = null;
            }
            return;
        }

        if (group == PERMISSION_GROUP_REQUEST_INSTALL_PACKAGES) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                if (activity.getPackageManager().canRequestPackageInstalls()) {
                    this.result.success(PERMISSION_STATUS_GRANTED);
                } else {
                    Intent intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES,
                            Uri.parse("package:" + activity.getPackageName()));
                    activity.startActivityForResult(intent, PERMISSION_CODE_REQUEST_INSTALL_PACKAGES);
                }
            } else {
                this.result.success(PERMISSION_STATUS_GRANTED);
            }
            return;
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            List<String> requestPermissions = new ArrayList<>();
            for (String manifestPermission : manifestPermissions) {
                if (activity.checkSelfPermission(manifestPermission) == PackageManager.PERMISSION_DENIED) {
                    requestPermissions.add(manifestPermission);
                }
            }
            if (requestPermissions.isEmpty()) {
                // 无需申请权限
                this.result.success(PERMISSION_STATUS_GRANTED);
                this.result = null;
            } else {
                activity.requestPermissions(requestPermissions.toArray(new String[0]), PERMISSION_CODE);
            }
        } else {
            this.result.success(PERMISSION_STATUS_GRANTED);
            this.result = null;
        }
    }

    private void shouldShowRequestPermissionRationale(MethodCall methodCall, MethodChannel.Result result) {
        Activity activity = registrar.activity();
        if (activity == null) {
            Log.d(TAG, "Unable to detect current Activity");
            result.error("shouldShowRequestPermissionRationale Error", "Unable to decect current Activity", null);
            return;
        }

        int group = methodCall.arguments();
        List<String> manifestPermissions = getManifestPermissions(group);
        if (manifestPermissions == null) {
            Log.d(TAG, "No android specific permissions needed for: " + group);
            result.success(false);
            return;
        }

        if (manifestPermissions.isEmpty()) {
            Log.d(TAG, "No permissions found in manifest for: " + group + " no need to show request rationale");
            result.success(false);
            return;
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            for (String permission : manifestPermissions) {
                if (activity.shouldShowRequestPermissionRationale(permission)) {
                    result.success(true);
                    return;
                }
            }
        }

        result.success(false);
    }

    private void openAppSetting(MethodChannel.Result result) {
        final Context context = registrar.activeContext();
        if (context == null) {
            Log.d(TAG, "Unable to detect current Activity or App Context");
            result.error("openAppSetting Error", "Unable to decect current Activity or App Context", null);
            return;
        }

        try {
            Intent intent = new Intent();
            intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
            intent.addCategory(Intent.CATEGORY_DEFAULT);
            intent.setData(Uri.parse("package:" + context.getPackageName()));
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
            intent.addFlags(Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS);

            context.startActivity(intent);
            result.success(true);
        } catch (Exception ex) {
            result.error("openAppSetting Error", ex.getMessage(), null);
        }
    }

    private List<String> getManifestPermissions(int group) {
        final ArrayList<String> permissionNames = new ArrayList<>();

        switch (group) {
            case PERMISSION_GROUP_CALENDAR:
                if (hasPermissionInManifest(Manifest.permission.READ_CALENDAR)) {
                    permissionNames.add(Manifest.permission.READ_CALENDAR);
                }
                if (hasPermissionInManifest(Manifest.permission.WRITE_CALENDAR)) {
                    permissionNames.add(Manifest.permission.WRITE_CALENDAR);
                }
                break;

            case PERMISSION_GROUP_CAMERA:
                if (hasPermissionInManifest(Manifest.permission.CAMERA)) {
                    permissionNames.add(Manifest.permission.CAMERA);
                }
                break;

            case PERMISSION_GROUP_CONTACTS:
                if (hasPermissionInManifest(Manifest.permission.READ_CONTACTS)) {
                    permissionNames.add(Manifest.permission.READ_CONTACTS);
                }
                if (hasPermissionInManifest(Manifest.permission.WRITE_CONTACTS)) {
                    permissionNames.add(Manifest.permission.WRITE_CONTACTS);
                }
                if (hasPermissionInManifest(Manifest.permission.GET_ACCOUNTS)) {
                    permissionNames.add(Manifest.permission.GET_ACCOUNTS);
                }
                break;

            case PERMISSION_GROUP_LOCATION:
                if (hasPermissionInManifest(Manifest.permission.ACCESS_COARSE_LOCATION)) {
                    permissionNames.add(Manifest.permission.ACCESS_COARSE_LOCATION);
                }

                if (hasPermissionInManifest(Manifest.permission.ACCESS_FINE_LOCATION)) {
                    permissionNames.add(Manifest.permission.ACCESS_FINE_LOCATION);
                }
                break;

            case PERMISSION_GROUP_MICROPHONE:
                if (hasPermissionInManifest(Manifest.permission.RECORD_AUDIO)) {
                    permissionNames.add(Manifest.permission.RECORD_AUDIO);
                }
                break;

            case PERMISSION_GROUP_PHONE:
                if (hasPermissionInManifest(Manifest.permission.READ_PHONE_STATE)) {
                    permissionNames.add(Manifest.permission.READ_PHONE_STATE);
                }
                if (hasPermissionInManifest(Manifest.permission.CALL_PHONE)) {
                    permissionNames.add(Manifest.permission.CALL_PHONE);
                }
                if (hasPermissionInManifest(Manifest.permission.READ_CALL_LOG)) {
                    permissionNames.add(Manifest.permission.READ_CALL_LOG);
                }
                if (hasPermissionInManifest(Manifest.permission.WRITE_CALL_LOG)) {
                    permissionNames.add(Manifest.permission.WRITE_CALL_LOG);
                }
                if (hasPermissionInManifest(Manifest.permission.ADD_VOICEMAIL)) {
                    permissionNames.add(Manifest.permission.ADD_VOICEMAIL);
                }
                if (hasPermissionInManifest(Manifest.permission.USE_SIP)) {
                    permissionNames.add(Manifest.permission.USE_SIP);
                }
                if (hasPermissionInManifest(Manifest.permission.PROCESS_OUTGOING_CALLS)) {
                    permissionNames.add(Manifest.permission.PROCESS_OUTGOING_CALLS);
                }
                break;

            case PERMISSION_GROUP_SENSORS:
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT_WATCH) {
                    if (hasPermissionInManifest(Manifest.permission.BODY_SENSORS)) {
                        permissionNames.add(Manifest.permission.BODY_SENSORS);
                    }
                }
                break;

            case PERMISSION_GROUP_SMS:
                if (hasPermissionInManifest(Manifest.permission.SEND_SMS)) {
                    permissionNames.add(Manifest.permission.SEND_SMS);
                }
                if (hasPermissionInManifest(Manifest.permission.RECEIVE_SMS)) {
                    permissionNames.add(Manifest.permission.RECEIVE_SMS);
                }
                if (hasPermissionInManifest(Manifest.permission.READ_SMS)) {
                    permissionNames.add(Manifest.permission.READ_SMS);
                }
                if (hasPermissionInManifest(Manifest.permission.RECEIVE_WAP_PUSH)) {
                    permissionNames.add(Manifest.permission.RECEIVE_WAP_PUSH);
                }
                if (hasPermissionInManifest(Manifest.permission.RECEIVE_MMS)) {
                    permissionNames.add(Manifest.permission.RECEIVE_MMS);
                }
                break;

            case PERMISSION_GROUP_STORAGE:
                if (hasPermissionInManifest(Manifest.permission.READ_EXTERNAL_STORAGE)) {
                    permissionNames.add(Manifest.permission.READ_EXTERNAL_STORAGE);
                }
                if (hasPermissionInManifest(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                    permissionNames.add(Manifest.permission.WRITE_EXTERNAL_STORAGE);
                }
                break;

            case PERMISSION_GROUP_IGNORE_BATTERY_OPTIMIZATIONS:
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M)
                    if (hasPermissionInManifest(Manifest.permission.REQUEST_IGNORE_BATTERY_OPTIMIZATIONS)) {
                        permissionNames.add(Manifest.permission.REQUEST_IGNORE_BATTERY_OPTIMIZATIONS);
                    }
                break;

            case PERMISSION_GROUP_SYSTEM_ALERT_WINDOW:
                if (hasPermissionInManifest(Manifest.permission.SYSTEM_ALERT_WINDOW)) {
                    permissionNames.add(Manifest.permission.SYSTEM_ALERT_WINDOW);
                }
                break;

            case PERMISSION_GROUP_REQUEST_INSTALL_PACKAGES:
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                    if (hasPermissionInManifest(Manifest.permission.REQUEST_INSTALL_PACKAGES)) {
                        permissionNames.add(Manifest.permission.REQUEST_INSTALL_PACKAGES);
                    }
                }
                break;
            default:
                return null;
        }

        return permissionNames;
    }

    private boolean hasPermissionInManifest(String permission) {
        if (manifestPermissions == null) {
            Context context = registrar.activeContext();
            if (context == null) {
                Log.d(TAG, "Unable to detect current Activity or App Context.");
                return false;
            }
            manifestPermissions = PermissionUtils.getManifestPermissions(context);
        }

        return manifestPermissions.contains(permission);
    }


    public void onActivityResult(int requestCode, int responseCode, Intent intent) {
        if (requestCode == PERMISSION_CODE_IGNORE_BATTERY_OPTIMIZATIONS) {
            if (result != null) {
                result.success(responseCode == Activity.RESULT_OK ? PERMISSION_STATUS_GRANTED : PERMISSION_STATUS_DENIED);
                result = null;
            }
        } else if (requestCode == PERMISSION_CODE_SYSTEM_ALERT_WINDOW) {
            if (result != null) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                    if (Settings.canDrawOverlays(registrar.activeContext())) {
                        result.success(PERMISSION_STATUS_GRANTED);
                    } else {
                        result.success(PERMISSION_STATUS_DENIED);
                    }
                }
                result = null;
            }
        } else if (requestCode == PERMISSION_CODE_REQUEST_INSTALL_PACKAGES) {
            if (result != null) {
                result.success(responseCode == Activity.RESULT_OK ? PERMISSION_STATUS_GRANTED : PERMISSION_STATUS_DENIED);
                result = null;
            }
        }
    }

    public void onRequestPermissionsResult(int id, String[] permissions, int[] grantResults) {
        if (id == PERMISSION_CODE) {
            if (result != null) {
                boolean isGranted = true;
                for (int i = 0; i < permissions.length; i++) {
                    if (grantResults[i] == PackageManager.PERMISSION_DENIED) {
                        isGranted = false;
                    }
                }

                result.success(isGranted ? PERMISSION_STATUS_GRANTED : PERMISSION_STATUS_DENIED);
                result = null;
            }
        }
    }
}
