package com.qr.flutter_toolkit.utils;

import android.app.Activity;
import android.graphics.Point;
import android.graphics.SurfaceTexture;
import android.hardware.Camera;
import android.os.Build;
import android.view.Surface;

import androidx.annotation.NonNull;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.Log;

public class CameraUtils {
    private final static String TAG = CameraUtils.class.getSimpleName();

    public static Camera openCamera(int cameraId) {
        try {
            return Camera.open(cameraId);
        } catch (Exception e) {
            Log.d(TAG, "Open Camera Error! CameraId: " + cameraId + " " + e.getMessage());
            return null;
        }
    }

    public static boolean startPreview(@NonNull Camera camera, @NonNull SurfaceTexture surfaceTexture) {
        try {
            camera.setPreviewTexture(surfaceTexture);
            camera.startPreview();
            return true;
        } catch (IOException e) {
            Log.d(TAG, "Camera StartPreview Error! " + e.getMessage());
            return false;
        }
    }

    public static boolean stopPreview(@NonNull Camera camera) {
        try {
            camera.stopPreview();
            return true;
        } catch (Exception e) {
            Log.d(TAG, "Camera StopPreview Error! " + e.getMessage());
            return false;
        }
    }

    public static List<Map<String, Object>> getAvailableCameras() {
        int numberOfCameras = Camera.getNumberOfCameras();
        List<Map<String, Object>> result = new ArrayList<>();
        for (int i = 0; i < numberOfCameras; i++) {
            Map<String, Object> map = new HashMap<>();
            Camera.CameraInfo cameraInfo = new Camera.CameraInfo();
            Camera.getCameraInfo(i, cameraInfo);
            map.put("facing", cameraInfo.facing);
            map.put("orientation", cameraInfo.orientation);
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
                map.put("canDisableShutterSound", cameraInfo.canDisableShutterSound);
            } else {
                map.put("canDisableShutterSound", false);
            }
            result.add(map);
        }
        return result;
    }

    public static boolean turnOnFlash(@NonNull Camera camera) {
        try {
            Camera.Parameters parameters = camera.getParameters();
            parameters.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH);
            camera.setParameters(parameters);
            return true;
        } catch (Exception e) {
            Log.d(TAG, "Open Flash Light Failure: " + e.getMessage());
            return false;
        }
    }

    public static boolean turnOffFlash(@NonNull Camera camera) {
        try {
            Camera.Parameters parameters = camera.getParameters();
            parameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
            camera.setParameters(parameters);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /***
     * 1920*1080的图片
     * @param sizeList
     * @param targetWidth = 1920
     * @param targetHeight = 1090
     * @return
     */
    public static Camera.Size findBestPreviewSizeValue(List<Camera.Size> sizeList, int targetWidth, int targetHeight) {
        Camera.Size bestSize = null;
        double targetRatio = (double) targetHeight / (double) targetWidth;
        double minDiff = 200.0;
        for (Camera.Size size : sizeList) {
            Log.d(TAG, "System Support Size, width: " + size.width + " height: " + size.height);
            if (size.width == targetWidth && size.height == targetHeight) {
                bestSize = size;
                break;
            }

            double supportedRatio = (double) size.height / (double) size.width;
            if (Math.abs(supportedRatio - targetRatio) < minDiff) {
                minDiff = Math.abs(supportedRatio - targetRatio);
                bestSize = size;
            }
        }

        if (bestSize != null) {
            Log.d(TAG, "BestSize width : " + bestSize.width + " height: " + bestSize.height);
        } else {
            Log.d(TAG, "Find BestSize Failure");
        }
        return bestSize;
    }

    public static void setCameraDisplayOrientation(@NonNull Camera camera, int facing, int rotation) {
        Camera.CameraInfo cameraInfo = new Camera.CameraInfo();
        Camera.getCameraInfo(facing, cameraInfo);
        int screenDegree = 0;
        switch (rotation) {
            case Surface.ROTATION_0:
                break;
            case Surface.ROTATION_90:
                screenDegree = 90;
                break;
            case Surface.ROTATION_180:
                screenDegree = 180;
                break;
            case Surface.ROTATION_270:
                screenDegree = 270;
                break;
        }

        int displayOrientation;
        if (cameraInfo.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
            displayOrientation = (cameraInfo.orientation + screenDegree) % 360;
            displayOrientation = (360 - displayOrientation) % 360;
        } else {
            displayOrientation = (cameraInfo.orientation - screenDegree + 360) % 360;
        }
        camera.setDisplayOrientation(displayOrientation);
        Log.d(TAG, "Camera DisplayOrientation : " + displayOrientation);
    }
}
