import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class CameraPreview extends StatefulWidget {
  final CameraController controller;

  const CameraPreview({Key key, @required this.controller}) : super(key: key);

  @override
  _CameraPreviewState createState() => _CameraPreviewState();
}

class _CameraPreviewState extends State<CameraPreview> {
  VoidCallback _listener;
  int _textureId;

  _CameraPreviewState() {
    _listener = () {
      final int newTextureId = widget.controller.textureId;
      if (newTextureId != _textureId) {
        setState(() {
          _textureId = newTextureId;
        });
      }
    };
  }

  @override
  void initState() {
    super.initState();
    _textureId = widget.controller.textureId;
    widget.controller.addListener(_listener);
  }

  @override
  void didUpdateWidget(CameraPreview oldWidget) {
    super.didUpdateWidget(oldWidget);
    oldWidget.controller.removeListener(_listener);
    _textureId = widget.controller.textureId;
    widget.controller.addListener(_listener);
  }

  @override
  void deactivate() {
    super.deactivate();
    widget.controller.removeListener(_listener);
  }

  @override
  Widget build(BuildContext context) {
    return _textureId == null
        ? Container()
        : Texture(
            textureId: _textureId,
          );
  }
}

class CameraDescription {
  static const int CAMERA_FACING_BACK = 0;
  static const int CAMERA_FACING_FRONT = 1;

  final int facing;
  final int orientation;
  final bool canDisableShutterSound;

  CameraDescription({
    @required this.facing,
    @required this.orientation,
    @required this.canDisableShutterSound,
  });
}

class CameraStatus {
  final bool isPreviewing;
  final bool isTakingPicture;
  final bool isRecordingVideo;
  final bool isError;
  final String errorString;
  final int cameraId;
  final int previewWidth;
  final int previewHeight;
  final int pictureWidth;
  final int pictureHeight;
  final int minFps;
  final int maxFps;

  double get aspectRatio => previewHeight / previewWidth;

  const CameraStatus({
    @required this.isPreviewing,
    @required this.isTakingPicture,
    @required this.isRecordingVideo,
    @required this.isError,
    @required this.errorString,
    @required this.cameraId,
    @required this.previewWidth,
    @required this.previewHeight,
    @required this.pictureWidth,
    @required this.pictureHeight,
    @required this.minFps,
    @required this.maxFps,
  });

  CameraStatus error({@required String errorString}) {
    return copyWith(
      isPreviewing: false,
      isTakingPicture: false,
      isRecordingVideo: false,
      isError: true,
      errorString: errorString,
    );
  }

  CameraStatus previewing({
    @required int cameraId,
    @required int previewWidth,
    @required int previewHeight,
    @required int pictureWidth,
    @required int pictureHeight,
    @required int minFps,
    @required int maxFps,
  }) {
    return copyWith(
      isPreviewing: true,
      isTakingPicture: false,
      isRecordingVideo: false,
      isError: false,
      errorString: null,
      cameraId: cameraId,
      previewWidth: previewWidth,
      previewHeight: previewHeight,
      pictureWidth: pictureWidth,
      pictureHeight: pictureHeight,
      minFps: minFps,
      maxFps: maxFps,
    );
  }

  CameraStatus copyWith({
    bool isPreviewing,
    bool isTakingPicture,
    bool isRecordingVideo,
    bool isError,
    String errorString,
    int cameraId,
    int previewWidth,
    int previewHeight,
    int pictureWidth,
    int pictureHeight,
    int minFps,
    int maxFps,
  }) {
    return CameraStatus(
      isPreviewing: isPreviewing ?? this.isPreviewing,
      isTakingPicture: isTakingPicture ?? this.isTakingPicture,
      isRecordingVideo: isRecordingVideo ?? this.isRecordingVideo,
      isError: isError ?? this.isError,
      errorString: errorString ?? this.errorString,
      cameraId: cameraId ?? this.cameraId,
      previewWidth: previewWidth ?? this.previewWidth,
      previewHeight: previewHeight ?? this.previewHeight,
      pictureWidth: pictureWidth ?? this.pictureWidth,
      pictureHeight: pictureHeight ?? this.pictureHeight,
      minFps: minFps ?? this.minFps,
      maxFps: maxFps ?? this.maxFps,
    );
  }
}

class CameraController extends ValueNotifier<CameraStatus> {
  static const MethodChannel _methodChannel =
      const MethodChannel("com.qr.flutter_toolkit/camera/methods");

  int _textureId;

  int get textureId => _textureId;

  CameraController()
      : super(CameraStatus(
          isPreviewing: false,
          isTakingPicture: false,
          isRecordingVideo: false,
          isError: false,
          errorString: null,
          cameraId: null,
          previewWidth: null,
          previewHeight: null,
          pictureWidth: null,
          pictureHeight: null,
          minFps: null,
          maxFps: null,
        ));

  Future<void> startPreview({
    int previewWidth = 1920,
    int previewHeight = 1080,
    int pictureWidth = 1920,
    int pictureHeight = 1080,
  }) async {
    try {
      var map = await _methodChannel.invokeMapMethod("startPreview", {
        "previewWidth": previewWidth,
        "previewHeight": previewHeight,
        "pictureWidth": pictureWidth,
        "pictureHeight": pictureHeight,
      });
      _textureId = map["textureId"];

      value = value.previewing(
        cameraId: map["cameraId"],
        previewWidth: map["previewWidth"],
        previewHeight: map["previewHeight"],
        pictureWidth: map["pictureWidth"],
        pictureHeight: map["pictureHeight"],
        minFps: map["minFps"],
        maxFps: map["maxFps"],
      );
    } catch (e) {
      print("Start Preview Fail! Reason: ${e.toString()}");
      value = value.error(errorString: e.toString());
    }
  }

  Future<void> stopPreview() async {
    try {
      await _methodChannel.invokeMethod("stopPreview");
      value = value.copyWith(
        isPreviewing: false,
        isTakingPicture: false,
        isError: false,
        errorString: null,
      );
    } catch (e) {
      print("Stop Preview Fail! Reason: ${e.toString()}");
      value = value.error(errorString: e.toString());
    }
  }

  Future<void> switchCamera({
    int previewWidth = 1920,
    int previewHeight = 1080,
    int pictureWidth = 1920,
    int pictureHeight = 1080,
  }) async {
    try {
      // 关闭预览
      value = value.copyWith(
        isPreviewing: false,
        isTakingPicture: false,
        isError: false,
        errorString: null,
      );

      var map = await _methodChannel.invokeMethod("switchCamera", {
        "previewWidth": previewWidth,
        "previewHeight": previewHeight,
        "pictureWidth": pictureWidth,
        "pictureHeight": pictureHeight,
      });
      _textureId = map["textureId"];

      // 开始预览
      value = value.previewing(
        cameraId: map["cameraId"],
        previewWidth: map["previewWidth"],
        previewHeight: map["previewHeight"],
        pictureWidth: map["pictureWidth"],
        pictureHeight: map["pictureHeight"],
        minFps: map["minFps"],
        maxFps: map["maxFps"],
      );
    } catch (e) {
      print("Switch Camera Fail! Reason: ${e.toString()}");
      value = value.error(errorString: e.toString());
    }
  }

  Future<String> takePicture({
    @required String dirPath,
    @required String fileName,
  }) async {
    value = value.copyWith(isTakingPicture: true);
    try {
      var path = await _methodChannel.invokeMethod("takePicture", {
        "dirPath": dirPath,
        "fileName": fileName,
      });
      value = value.copyWith(isTakingPicture: false);
      return path;
    } catch (e) {
      print("TakePicture Fail! Reason: " + e.toString());
      value = value.error(errorString: e.toString());
      return null;
    }
  }

  Future<void> startRecording() async {
    try {
      value = value.copyWith(isRecordingVideo: true);
      await _methodChannel.invokeMethod("startRecording");
    } catch (e) {
      print(e);
      value = value.copyWith(isRecordingVideo: false);
    }
  }

  Future<void> stopRecording() async {
    value = value.copyWith(isRecordingVideo: false);
    try {
      await _methodChannel.invokeMethod("stopRecording");
    } catch (e) {
      print(e);
    }
  }

  Future<List<CameraDescription>> getAvailableCameras() async {
    List<Map<dynamic, dynamic>> cameras = await _methodChannel
        .invokeListMethod<Map<dynamic, dynamic>>("getAvailableCameras");
    return cameras.map((camera) {
      return CameraDescription(
        facing: camera["facing"],
        orientation: camera["orientation"],
        canDisableShutterSound: camera["canDisableShutterSound"],
      );
    }).toList();
  }

  @override
  void dispose() {
    stopPreview();
    super.dispose();
  }
}
